/* 
 * Mach Operating System
 * Copyright (c) 1992 Carnegie-Mellon University
 * All rights reserved.  The CMU software License Agreement specifies
 * the terms and conditions for use and redistribution.
 */
/*
 * HISTORY
 * $Log:	proxy_tcp.h,v $
 * Revision 2.2  93/08/07  17:04:02  mrt
 * 	Created.
 * 	[93/06/12            cmaeda]
 * 
 */

/*
 * Proxy sockets have the following state which is kept
 * in the inp_ppcb field of the socket inpcb struct.
 */

/*
 * Tcp control block, one per tcp; fields:
 *
 * We use two mbufs for this (pt_tcb is in a separate mbuf)
 * since it won't fit into one.  Also, pt_tcb comes first so
 * that we can reuse parts of the tcp code and so that netstat
 * will work.
 *
 * sizeof(struct tcpcb) == 108
 */

struct proxy_tcpcb2 {
	u_long		pt2_state;
	mach_port_t	pt2_control_port;	/* send control message here */
	mach_port_t	pt2_network_port; 	/* network device port */
	mach_port_t	pt2_pfilter_port;	/* packet filter port */
	struct proxy_tcpcb *pt2_tcb;
};

/* assert(sizeof(struct proxy_tcpcb) == MLEN) */
struct proxy_tcpcb {
	struct tcpcb		pt_tcb;		/* pass this to tcp code */
	struct proxy_tcpcb2 	*pt_tcb2;	/* proxy state */
#define pt_state	pt_tcb2->pt2_state
#define pt_control_port pt_tcb2->pt2_control_port
#define pt_network_port pt_tcb2->pt2_network_port
#define pt_pfilter_port pt_tcb2->pt2_pfilter_port
};

#define	intoptcpcb(ip)	((struct proxy_tcpcb *)(ip)->inp_ppcb)
#define	sotoptcpcb(so)	(intoptcpcb(sotoinpcb(so)))

#define PTS_LOCAL	0x0001	/* just like a normal tcp conn, tcb is valid */
#define PTS_REMOTE	0x0002	/* handled in peer task, tcb is invalid */
#define PTS_DIDFORK	0x0004	/* forked, will stay in UX */

/*
 * Structure of a tcp tcb shipped in a message.
 */
struct proxy_tcb {
	short	t_state;		/* state of this connection */
	short	t_timer[TCPT_NTIMERS];	/* tcp timers */
	short	t_rxtshift;		/* log(2) of rexmt exp. backoff */
	short	t_rxtcur;		/* current retransmit value */
	short	t_dupacks;		/* consecutive dup acks recd */
	u_short	t_maxseg;		/* maximum segment size */
	char	t_force;		/* 1 if forcing out a byte */
	u_char	t_flags;
/*
 * The following fields are used as in the protocol specification.
 * See RFC783, Dec. 1981, page 21.
 */
/* send sequence variables */
	tcp_seq	snd_una;		/* send unacknowledged */
	tcp_seq	snd_nxt;		/* send next */
	tcp_seq	snd_up;			/* send urgent pointer */
	tcp_seq	snd_wl1;		/* window update seg seq number */
	tcp_seq	snd_wl2;		/* window update seg ack number */
	tcp_seq	iss;			/* initial send sequence number */
	u_short	snd_wnd;		/* send window */
/* receive sequence variables */
	u_short	rcv_wnd;		/* receive window */
	tcp_seq	rcv_nxt;		/* receive next */
	tcp_seq	rcv_up;			/* receive urgent pointer */
	tcp_seq	irs;			/* initial receive sequence number */
/*
 * Additional variables for this implementation.
 */
/* receive variables */
	tcp_seq	rcv_adv;		/* advertised window */
/* retransmit variables */
	tcp_seq	snd_max;		/* highest sequence number sent
					 * used to recognize retransmits
					 */
/* congestion control (for slow start, source quench, retransmit after loss) */
	u_short	snd_cwnd;		/* congestion-controlled window */
	u_short snd_ssthresh;		/* snd_cwnd size threshhold for
					 * for slow start exponential to
					 * linear switch */
/*
 * transmit timing stuff.
 * srtt and rttvar are stored as fixed point; for convenience in smoothing,
 * srtt has 3 bits to the right of the binary point, rttvar has 2.
 * "Variance" is actually smoothed difference.
 */
	short	t_idle;			/* inactivity time */
	short	t_rtt;			/* round trip time */
	tcp_seq	t_rtseq;		/* sequence number being timed */
	short	t_srtt;			/* smoothed round-trip time */
	short	t_rttvar;		/* variance in round-trip time */
	u_short	max_sndwnd;		/* largest window peer has offered */
/* out-of-band data */
	char	t_oobflags;		/* have some */
	char	t_iobc;			/* input character */
};
	
